<?php declare(strict_types=1);

namespace EuKit\Base;

use EuKit\Base\Interfaces\ViewModel;
use Infomaker\Everyware\Base\OpenContent\QueryBuilder;
use Infomaker\Everyware\Base\OpenContent\OpenContentProvider;
use Infomaker\Everyware\Support\Date;

class ArchivePageTemplate implements ViewModel
{
    /**
     * @var Date
     */
    private $startTime;

    /**
     * @var Date
     */
    private $endTime;

    public function __construct(int $year, int $month = null, int $day = null)
    {
        $targetDate = Date::create($year, $month !== 0 ? $month : null, $day !== 0 ? $day : null);

        $this->setTimeSpan($targetDate->isFuture() ? Date::now() : $targetDate, $month, $day);
    }

    public function getViewData(): array
    {
        $provider = OpenContentProvider::setup([
            'q' => $this->buildTimeSpanQuery('Pubdate'),
            'contenttypes' => ['Article'],
            'sort.indexfield' => 'Pubdate',
            'sort.Pubdate.ascending' => 'false',
            'limit' => 30
        ]);

        $articles = $provider->queryWithRequirements();

        return [
            'title' => 'Archive from: ' . $this->startTime->toDateString() . ' To: ' . $this->endTime->toDateString(),
            'articles' => array_map(function (\OcArticle $article) {
                return [
                    'headline' => $article->get_value('headline'),
                    'permalink' => $article->get_permalink(),
                    'published' => Date::createFromOcString($article->get_value('pubdate'))->diffForHumans()
                ];
            }, $articles)
        ];
    }

    private function setTimeSpan(Date $targetDate, $month = null, $day = null): void
    {
        $this->startTime = $targetDate;
        $this->endTime = $this->startTime->isToday() ? Date::now() : $this->startTime->copy()->endOfDay();

        if ($day) {
            $this->startTime->startOfDay();

            return;
        }

        if ($month) {
            $this->startTime->startOfMonth();

            if ( ! $this->startTime->isCurrentMonth()) {
                $this->endTime->endOfMonth();
            }

            return;
        }

        $this->startTime->startOfYear();

        if ( ! $this->startTime->isCurrentYear()) {
            $this->endTime->endOfYear();
        }
    }

    private function buildTimeSpanQuery($property): string
    {
        return QueryBuilder::dateSpanSearch(
            $property,
            $this->startTime->copy()->toOcString(),
            $this->endTime->copy()->toOcString()
        );
    }
}
