<?php declare(strict_types=1);

namespace EuKit\Base;

use Infomaker\Everyware\Base\Interfaces\NewsItemPresentation;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Everyware\Support\Storage\Cache;
use Infomaker\Everyware\Twig\View;

class ArticleBodyPresentation implements NewsItemPresentation
{
    private $relatedArticles;

    public function __construct(NewsMLArticle $article)
    {
        $this->relatedArticles = [];

        foreach ($article->get('articles', []) as $relatedArticle) {
            if ($relatedArticle->permalink !== null) {
                $this->relatedArticles[$relatedArticle->uuid] = $relatedArticle->permalink;
            }
        }
    }

    protected function element(Item $item): string
    {
        return $this->generateArticlePart('element', [
            'type' => $item->getType(),
            'content' => htmlspecialchars_decode($item->get('content')),
        ]);
    }

    private function image(ImageItem $item): string
    {
        $image = new Image($item);
        return $this->generateArticlePart('image', $image->getViewData());
    }

    private function table(Item $item): string
    {
        return $this->generateArticlePart('table', $item->toArray());
    }

    private function htmlEmbed(Item $item)
    {
        //return $item->get('content');
        return $this->generateArticlePart('html-embed', $item->toArray());
    }

    private function imageGallery(Item $item): string
    {
        $images = array_map([&$this, 'image'], (array)$item->get('images'));

        $item->set('images', $images);

        return $this->generateArticlePart('gallery', $item->toArray());
    }

    private function contentPart(Item $item): string
    {
        $contentPartText = array_map([&$this, 'element'], (array)$item->get('text'));

        $item->set('text', implode('', $contentPartText));

        return $this->generateArticlePart('content-part', $item->toArray());
    }

    private function relatedArticle(Item $item): string
    {
        if (isset($this->relatedArticles[$item->uuid])) {
            $item->set('permalink', $this->relatedArticles[$item->uuid]);

            return $this->generateArticlePart('related-article', $item->toArray());
        }

        return '';
    }

    private function pdf(Item $item): string
    {
        return $this->generateArticlePart('pdf', [
            'url' => $item->get('url'),
            'title' => $item->get('title'),
        ]);
    }

    private function youtube(Item $item): string
    {
        return $this->generateArticlePart('youtube', $item->toArray());
    }

    private function review(Item $item): string
    {
        $text = array_map([&$this, 'element'], (array)$item->get('text'));

        $item->set('text', implode('', $text));

        return $this->generateArticlePart('review', $item->toArray());
    }

    private function youplay(Item $item): string
    {
        return $this->generateArticlePart('youplay', $item->toArray());
    }

    private function socialEmbed(Item $item): ?string
    {
        $links = $item->get('content');

        if (empty($links)) {
            return '';
        }

        if ( ! ($linkItem = array_shift($links)) instanceof Item) {
            return '';
        }
        switch ($linkItem->getType()) {
            case 'x-im/tweet':
                return $this->twitter($linkItem);
                break;
            case 'x-im/facebook-post':
                return $this->facebook($linkItem);
                break;
            case 'x-im/instagram':
                return $this->instagram($linkItem);
                break;
            default:
                return '';
                break;
        }
    }

    private function instagram(Item $item): string
    {
        $embed_url = $item->embed_url ?? $item->url ?? '';
        $key = md5("instagram_{$embed_url}");

        if (($content = Cache::get($key)) === false) {
            try {
                if (isset($item->url)) {
                    $content = file_get_contents('https://api.instagram.com/oembed/?url=' . $item->url);
                }
            } catch (\Exception $e) {
                return '';
            }

            Cache::set($key, $content, 5 * MINUTE_IN_SECONDS);
        }

        return $this->generateArticlePart('instagram', [
            'content' => json_decode($content)->html
        ]);
    }

    private function facebook(Item $item): string
    {
        return $this->generateArticlePart('facebook', [
            'url' => $item->url ?? ''
        ]);
    }

    private function twitter(Item $item): string
    {
        return $this->generateArticlePart('twitter', [
            'url' => $item->url ?? ''
        ]);
    }

    private function generateArticlePart($part, $data): string
    {
        try {
            return View::generate("@base/article/{$part}", $data);
        } catch (\Twig_Error $e) {
            NewRelicLog::error('Failed to generate article body part', $e);
        }

        return '';
    }

    public function generateItem(string $type, Item $item): string
    {
        switch ($type) {
            case 'x-im/youtube':
                return $this->youtube($item);
            case 'x-im/youplay':
                return $this->youplay($item);
            case 'x-im/image':
                return $item instanceof ImageItem ? $this->image($item) : '';
            case 'x-im/link':
                return implode('', $item->toArray());
            case 'x-im/article':
                return $this->relatedArticle($item);
            case 'x-gm/review':
                return $this->review($item);
            case 'x-im/socialembed':
                return $this->socialEmbed($item);
            case 'x-im/content-part':
                return $this->contentPart($item);
            case 'x-im/htmlembed':
                return $this->htmlEmbed($item);
            case 'x-im/imagegallery':
                return $this->imageGallery($item);
            case 'x-im/table':
                return $this->table($item);
            case 'x-im/pdf':
                return $this->pdf($item);
            default:
                return $this->element($item);

        }
    }
}
